;------------------------------------------------------------------------------
; sample.asm
;
; This is a sample program that demonstrates connecting a 14-bit PIC
; microcontroller to the uM-FPU floating point coprocessor. It displays the
; results of a simple conversion by sending output to the asynchronous port.
;
; The list, #include, and CONFIG settings should be changed if another PIC
; device with compatible features is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V2.0 - November 22, 2004
; changes:
;	V2.0 - November 22, 2004
;	  - update for uM-FPU V2.0
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF 

	#include umfpu.inc				;um-FPU definitions
	extern	print_setup, print_byte, print_crlf, print_string
	extern	print_version, print_float

;-------------------- uM-FPU register definitions -----------------------------

#define	DiameterIn		1			;diameter in inches	(uM-FPU register)
#define	Circumference	2			;circumference		(uM-FPU register)
#define	Area			3			;area				(uM-FPU register)
#define	F2_0			4			;constant 2.0		(uM-FPU register)
#define	F2_54			5			;constant 2.54		(uM-FPU register)

;-------------------- variable definitions ------------------------------------
	udata

diameterCm		res	1				;loop counter

;-------------------- string definitions --------------------------------------
; lower byte of stringTable address must be 00
; stringTable can have up to 255 bytes of data
; all strings must be zero terminated

STRINGS	code						;store strings on page 1 (PROG2)
	global	stringTable
stringTable
	addwf	PCL,f					;computed goto for strings

startMessage
	dt	0x0D, 0x0A
	dt	0x0D, 0x0A, "Conversion Example"
	dt	0x0D, 0x0A, "------------------", 0x0D, 0x0A, 0
errorMessage:
	dt	"uM-FPU not detected", 0
diameterMessage1
	dt 0x0D, 0x0A, 0x0A, "Diameter (cm.):      25", 0
diameterMessage2
	dt 0x0D, 0x0A, "Diameter (in.):      ", 0
circumferenceMessage
	dt 0x0D, 0x0A, "Circumference (in.): ", 0
areaMessage
	dt 0x0D, 0x0A, "Area (sq.in.):       ", 0
doneMessage
	dt 0x0D, 0x0A, 0x0A, "Done.", 0x0D, 0x0A, 0
	
;-------------------- reset and interrupt vector ------------------------------

STARTUP	code
	nop									;reset vector
  	goto    main
	nop
	nop
	goto	isr							;interrupt vector

;-------------------- interrupt service routine -------------------------------

PROG1	code
isr
	retfie								;(no interrupts used)

;-------------------- main ----------------------------------------------------

main		
 	call		print_setup				;initialize the serial port

	;--------------------------------------------------------------------------
	; The following example takes an integer value representing the diameter
	; of a circle in centimeters, converts the value to inches, and calculates
	; the circumference and area in inches and square inches.
	;--------------------------------------------------------------------------

	movlw		LOW startMessage		;display startup string
	call		print_string

	;------------------
	; Reset the uM-FPU.
	;------------------

	call		fpu_reset				;reset the uM-FPU
	btfsc		STATUS, Z				;check status
	goto		main2
	movlw		LOW errorMessage		;print error message if reset failed
	call		print_string
	goto		done

main2
	call		print_version			;print uM-FPU version number

	;--------------------------------------------------------------------------
	; Load constants for later user.
	;--------------------------------------------------------------------------

	movlw		SELECTA + F2_0			;store value for 2.0
	call		fpu_sendByte
	movlw		LOADBYTE
	call		fpu_sendByte
	movlw		.2
	call		fpu_sendByte
	movlw		FSET
	call		fpu_sendByte

	movlw		FWRITEA + F2_54			;store value for 2.54
	call		fpu_sendByte
	movlw		0x40
	call		fpu_sendByte
	movlw		0x22
	call		fpu_sendByte
	movlw		0x8F
	call		fpu_sendByte
	movlw		0x5C
	call		fpu_sendByte

	;--------------------------------------------------------------------------
	; Get diameter in centimeters. The value would typically come from a sensor
	; reading, but in this example an assumed value of 25 is used.
	;--------------------------------------------------------------------------

	movlw		.25						;diameterCm = 25
	movwf		diameterCm

	movlw		LOW diameterMessage1
	call		print_string			;display diameter in cm

	;--------------------------------------------------------------------------
	; DiameterIn = diameterCm / 2.54
	;--------------------------------------------------------------------------

	movlw		SELECTA + DiameterIn	;select DiameterIn as register A			
	call		fpu_sendByte

	movlw		LOADBYTE				;load diameterCm to register 0
	call		fpu_sendByte			; and convert to float
	movf		diameterCm, w
	call		fpu_sendByte

	movlw		FSET					;DiameterIn = register 0
	call		fpu_sendByte				

	movlw		FDIV + F2_54			;divide by 2.54
	call		fpu_sendByte

	movlw		LOW diameterMessage2
	call		print_string			;display diameter in inches
	call		print_float

	;--------------------------------------------------------------------------
	; Circumference = DiameterIn * pi
	;--------------------------------------------------------------------------

	movlw		SELECTA + Circumference	;select Circumference as register A
	call		fpu_sendByte

	movlw		FSET + DiameterIn		;Circumference = DiameterIn
	call		fpu_sendByte
	
	movlw		XOP						;load value of pi to register 0
	call		fpu_sendByte
	movlw		LOADPI
	call		fpu_sendByte

	movlw		FMUL					;Circumference = Circumference * pi
	call		fpu_sendByte

	movlw		LOW circumferenceMessage
	call		print_string			;display Circumference in inches
	call		print_float

	;--------------------------------------------------------------------------
	; Area = (DiameterIn / 2) ** 2 * pi
	;--------------------------------------------------------------------------

	movlw		SELECTA + Area			;select Area as register A
	call		fpu_sendByte

	movlw		FSET + DiameterIn		;Area = DiameterIn
	call		fpu_sendByte

	movlw		FDIV + F2_0				;divide by 2.0
	call		fpu_sendByte

	movlw		FMUL + Area				;Area = Area * Area
	call		fpu_sendByte

	movlw		XOP						;load value of pi to register 0
	call		fpu_sendByte
	movlw		LOADPI
	call		fpu_sendByte

	movlw		FMUL					;Area = Area * pi
	call		fpu_sendByte

	movlw		LOW areaMessage
	call		print_string			;display Area in inches
	call		print_float

	;--------------------------------------------------------------------------
	; End of program.
	;--------------------------------------------------------------------------

	movlw		LOW doneMessage
	call		print_string			;display diameter in inches

done
	goto		done					;end of program

	end
