;------------------------------------------------------------------------------
; fpuhw_4.asm
;
; This module provides low-level interface routines for connecting a PIC16F877
; microcontroller to the uM-FPU floating point coprocessor.  It uses the
; PIC16F877 Master Synchronous Serial Port configured for SPI mode, and assumes
; a clock speed of 4 MHz. The hardware SPI is configured using a clock source of
; 1.00 MHz (FOSC/4). (There is also a 20 MHz version of this file.)
;
; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V2.0 November 21, 2004
; changes:
;	V2.0 November 21, 2004
;	  - updated for uM-FPU V2.0
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list	p=16f877					;list directive to define processor
	#include <p16f877.inc>				;processor specific definitions
	
	extern	delay_ms, delay

;-------------------- variable definitions ------------------------------------
		udata
dataBits		res	1					;data bits

#define	FPU_CLOCK		PORTC, 3		;clock (uM-FPU SCLK)
#define FPU_CLOCK_DIR	TRISC, 3		;direction bit
#define	FPU_DATAIN		PORTC, 4		;data input(uM-FPU SOUT)
#define FPU_DATAIN_DIR	TRISC, 4		;direction bit
#define	FPU_DATAOUT		PORTC, 5		;data output (uM-FPU SIN)
#define FPU_DATAOUT_DIR	TRISC, 5		;direction bit

#define WRITEBYTE_DELAY	0				;additional delay for write data period
										;(to ensure write data period of 15 usec)
#define READBYTE_DELAY	0				;additional delay for read byte delay
										;(to ensure read byte delay of 15 usec)
#define READ_DELAY		.36				;read setup delay (36 * 5 = 180 usec)

#define FSYNC		0xF0				;synchronization opcode
#define SYNC_CHAR	0x5C				;sync character

PROG1	code

;-------------------- fpu_reset -----------------------------------------------
; Resets the uM-FPU.
;	input:	none
;	output:	Z flag = 1, Reset successful
;			Z flag = 0, Reset failed 
;------------------------------------------------------------------------------

		global		fpu_reset
fpu_reset
	banksel		TRISC
	bcf			FPU_CLOCK_DIR			;configure FPU_CLOCK as output
	bsf			FPU_DATAIN_DIR			;configure FPU_DATAIN as input
	bcf			FPU_DATAOUT_DIR			;configure FPU_DATAOUT as output
	banksel		PORTC

	bcf			FPU_CLOCK				;reset the uM-FPU
	bcf			FPU_DATAOUT
	bsf			FPU_CLOCK
	movlw		.100					;clock high for 500 usec (100 x 5 usec)
	call		delay
	bcf			FPU_CLOCK
	movlw		.8						;wait for reset to finish
	call		delay_ms

	bcf			SSPCON, SSPEN			;disable the SPI module
	movlw		B'00000000'				;CKP=0, clock idle low, clk=fosc/4
	movwf		SSPCON
	movlw		B'01000000'				;SMP=0, CKE=1, sample on rising edge
	banksel		SSPSTAT
	movwf		SSPSTAT
	banksel		SSPCON
	bsf			SSPCON, SSPEN			;enable the SPI module

	movlw		FSYNC					;confirm communications
	call		fpu_sendByte
	call		fpu_readDelay
	call		fpu_readByte
	sublw		SYNC_CHAR
	return

;-------------------- fpu_wait ------------------------------------------------
; Waits for ready condition.
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_wait
fpu_wait	
	btfsc		FPU_DATAIN				;wait for ready condition
	goto		fpu_wait
	return

;-------------------- fpu_sendByte --------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendByte
fpu_sendByte
	movwf		SSPBUF					;send byte to SPI

	banksel		SSPSTAT					;wait for transmit to finish
fpu_sendByte2
	btfss		SSPSTAT, BF
	goto		fpu_sendByte2
	banksel		SSPBUF

#if	WRITEBYTE_DELAY != 0
	movlw		WRITEBYTE_DELAY			;additional delay for write data period
	call		delay	
#endif
	return

;-------------------- fpu_readByte --------------------------------------------
; Gets 8-bit byte from uM-FPU.
;	input:	none
;	output:	W register	- 8-bit value received from uM-FPU
;			dataBits	- 8-bit value received from uM-FPU
;			Z flag 		- set according to value of W register
;------------------------------------------------------------------------------

	global		fpu_readByte
fpu_readByte
	clrf		SSPBUF					;send zero to SPI

	banksel		SSPSTAT					;wait for receive to finish
fpu_readByte2
	btfss		SSPSTAT, BF
	goto		fpu_readByte2
	banksel		SSPBUF

#if	READBYTE_DELAY != 0
	movlw		READBYTE_DELAY			;additional delay for read data delay
	call		delay	
#else
	nop
#endif
	
	movf		SSPBUF, w				;get the input byte
	movwf		dataBits
	return

;-------------------- fpu_readDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_readDelay
fpu_readDelay
	movlw		READ_DELAY				;delay before read
	call		delay	
	return

	end
