;------------------------------------------------------------------------------
; graph.asm
;
; This is a sample program that demonstrates connecting a 14-bit PIC
; microcontrollers to the uM-FPU floating point coprocessor. It displays a
; simple graph of the sine and cosine functions, by sending output to the
; asynchronous port.
;
; The list, #include, and CONFIG settings should be changed if another PIC
; device with compatible features is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V2.0 November 21, 2004
; changes:
;	V2.0 November 21, 2004
;	  - updated for uM-FPU V2.0
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f877				;list directive to define processor
	#include <p16f877.inc>			;processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 

	#include umfpu.inc				;um-FPU definitions
	extern	print_setup, print_byte, print_crlf, print_string
	extern	print_version, print_floatFormat

;-------------------- uM-FPU register definitions -----------------------------

#define	Angle			1			;angle			(uM-FPU register 1)
#define	Point			2			;point on graph	(uM-FPU register 2)
#define	Pi				3			;constant pi	(uM-FPU register 3)
#define	F10_0			4			;constant 10.0	(uM-FPU register 4)
#define	F20_0			5			;constant 20.0	(uM-FPU register 5)

;-------------------- variable definitions ------------------------------------
	udata

xval	res	1						;x value for graph
yval	res	1						;y value for graph
pval	res	1						;point value for graph

;-------------------- string definitions --------------------------------------
; lower byte of stringTable address must be 00
; stringTable can have up to 255 bytes of data
; all strings must be zero terminated

STRINGS	code
	global	stringTable
stringTable
	addwf	PCL,f					;computed goto for strings

startMessage
	dt	0x0D, 0x0A
	dt	0x0D, 0x0A, "Graphdemo V1.0"
	dt	0x0D, 0x0A, "--------------", 0x0D, 0x0A, 0
errorMessage
	dt	"uM-FPU not detected", 0
sinString
	dt 0x0D, 0x0A, 0x0D, 0x0A, "Graph of sine", 0
cosString
	dt 0x0D, 0x0A, 0x0D, 0x0A, "Graph of cosine", 0
axisString
	dt 0x0D, 0x0A, "-1        0         1"
	dt 0x0D, 0x0A, "---------------------", 0x0D, 0x0A, 0
doneMessage
	dt 0x0D, 0x0A, 0x0A, "Done.", 0x0D, 0x0A, 0
	
;-------------------- reset and interrupt vector ------------------------------

STARTUP	code
	nop								;reset vector
  	goto    main
	nop
	nop
	goto	isr						;interrupt vector

;-------------------- interrupt service routine -------------------------------

PROG	code
isr
	retfie							;(no interrupts used)

;-------------------- main ----------------------------------------------------

main		
 	call		print_setup			;initialize the serial port

	;--------------------------------------------------------------------------
	; The following example draws a simple graph of the sine function calculated
	; at 20 points from x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW startMessage	;display startup string
	call		print_string

	;--------------------------------------------------------------------------
	; Reset the uM-FPU.
	;--------------------------------------------------------------------------

	call		fpu_reset				;reset the uM-FPU
	btfsc		STATUS, Z				;check status
	goto		main2
	movlw		LOW errorMessage		;print error message if reset failed
	call		print_string
	goto		done

main2
	call		print_version			;print uM-FPU version number

	;--------------------------------------------------------------------------
	; Load constants for later user.
	;--------------------------------------------------------------------------

	call		fpu_wait
	movlw		SELECTA+F10_0			;set constant 10.0
	call		fpu_sendByte
	movlw		LOADBYTE
	call		fpu_sendByte
	movlw		.10
	call		fpu_sendByte
	movlw		FSET
	call		fpu_sendByte

	movlw		SELECTA+F20_0			;set constant 20.0
	call		fpu_sendByte			;(F20_0 = F10_0 + F10_0)
	movlw		FSET+F10_0
	call		fpu_sendByte
	movlw		FADD+F10_0
	call		fpu_sendByte
	
	;--------------------------------------------------------------------------
	; Draws a simple graph of the sine function calculated at 20 points from
	; x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW sinString			;display title of graph
	call		print_string
	movlw		LOW axisString			;display axis
	call		print_string

	clrf		xval					;clear x value

sinLoop
	call		fpu_wait
	movlw		SELECTA+Angle			;Angle = sin((pi * xval * 2) / 20.0)
	call		fpu_sendByte
	movlw		XOP
	call		fpu_sendByte
	movlw		LOADPI
	call		fpu_sendByte
	movlw		FSET
	call		fpu_sendByte
	movlw		LOADBYTE
	call		fpu_sendByte
	movf		xval, w
	addwf		xval, w
	call		fpu_sendByte
	movlw		FMUL
	call		fpu_sendByte
	movlw		FDIV+F20_0
	call		fpu_sendByte
	movlw		SIN
	call		fpu_sendByte

	call		graph					;graph the point

	incf		xval, f					;increment the y value
	movf		xval, w					;check if done
	sublw		.20
	btfsc		STATUS, C
	goto		sinLoop					;no, then display next y value

	;--------------------------------------------------------------------------
	; Draws a simple graph of the cosine function calculated at 20 points from
	; x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW cosString			;display title of graph
	call		print_string
	movlw		LOW axisString			;display axis
	call		print_string

	clrf		xval					;clear x value

cosLoop
	movlw		SELECTA+Angle			;Angle = cos((pi * xval * 2) / 20.0)
	call		fpu_sendByte
	movlw		XOP
	call		fpu_sendByte
	movlw		LOADPI
	call		fpu_sendByte
	movlw		FSET+0
	call		fpu_sendByte
	movlw		LOADBYTE
	call		fpu_sendByte
	movf		xval, w
	addwf		xval, w
	call		fpu_sendByte
	movlw		FMUL+0
	call		fpu_sendByte
	movlw		FDIV+F20_0
	call		fpu_sendByte
	movlw		COS
	call		fpu_sendByte

	call		graph					;graph the point

	incf		xval, f					;increment the y value
	movf		xval, w					;check if done
	sublw		.20
	btfsc		STATUS, C
	goto		cosLoop					;no, then display next y value
	
	;--------------------------------------------------------------------------
	; End of program.
	;--------------------------------------------------------------------------

	movlw		LOW doneMessage
	call		print_string			;display diameter in inches

done
	goto		done					;end of program

;-------------------- graph -----------------------------------------------
; draw simple graph of values
;--------------------------------------------------------------------------

graph
	movlw		SELECTA+Point			;Point = Angle * 10.0
	call		fpu_sendByte
	movlw		FSET+Angle
	call		fpu_sendByte
	movlw		FMUL+F10_0
	call		fpu_sendByte

	movlw		ROUND					;Reg0 = fix(round(Point))
	call		fpu_sendByte
	movlw		FIX
	call		fpu_sendByte

	call		fpu_wait				;wait until uM-FPU is ready
	movlw		FREAD					;read the value from Reg0
	call		fpu_sendByte
	call		fpu_readDelay
	call		fpu_readByte
	call		fpu_readByte
	call		fpu_readByte
	call		fpu_readByte			;(only keep lower 8 bits)

	addlw		.10						;add offset to axis
	movwf		pval

	clrf		yval					;clear the y value

graphLoop:
	movf		yval, w					;if y value = point value, display asterisk
	subwf		pval, w
	btfss		STATUS, Z
	goto		graph2
	movlw		'*'
	goto		graph4

graph2
	movf		yval, w					;elseif y value = axis, display vertical bar
	sublw		.10
	btfss		STATUS, Z
	goto		graph3
	movlw		'|'
	goto		graph4

graph3
	movlw		' '						;else, display blank

graph4
	call		print_byte				;display the byte

	incf		yval, f					;increment the y value
	movf		yval, w
	sublw		.20						;check if done
	btfsc		STATUS, C
	goto		graphLoop				;no, then display next y value

	movlw		' '						;display space
	call		print_byte

	movlw		SELECTA+Angle			;display Angle value in 8.5 format
	call		fpu_sendByte	
	movlw		.85
	call		print_floatFormat

	call		print_crlf				;display CR, LF and exit
	return	

	end
